@@ -37,6 +37,8 @@ Tile caching (creation, caching and retrieval of tiles).
 
 from __future__ import with_statement
 
+from contextlib import contextmanager
+
 from mapproxy.grid import MetaGrid
 from mapproxy.image import merge_images
 from mapproxy.image.tile import TileSplitter
@@ -76,18 +78,47 @@ class TileManager(object):
             elif any(source.supports_meta_tiles for source in sources):
                 raise ValueError('meta tiling configured but not supported by all sources')
     
+    @contextmanager
+    def session(self):
+        """
+        Context manager for access to the cache. Cleans up after usage
+        for connection based caches.
+        
+        >>> with tile_manager.session(): #doctest: +SKIP
+        ...    tile_manager.load_tile_coords(tile_coords)
+        
+        """
+        yield
+        self.cleanup()
+    
+    def cleanup(self):
+        if hasattr(self.cache, 'cleanup'):
+            self.cache.cleanup()
+    
     def load_tile_coord(self, tile_coord, with_metadata=False):
-        return self.load_tile_coords([tile_coord], with_metadata)[0]
+        tile = Tile(tile_coord)
+        self.cache.load_tile(tile, with_metadata)
+        
+        if tile.coord is not None and not self.is_cached(tile):
+            # missing or staled
+            creator = self.creator()
+            created_tiles = creator.create_tiles([tile])
+            for created_tile in created_tiles:
+                if created_tile.coord == tile_coord:
+                    return created_tile
+        
+        return tile    
     
     def load_tile_coords(self, tile_coords, with_metadata=False):
         tiles = TileCollection(tile_coords)
         uncached_tiles = []
         
+        # load all in batch
+        self.cache.load_tiles(tiles, with_metadata)
+        
         for tile in tiles:
-            # TODO cache eviction
-            if self.is_cached(tile):
-                self.cache.load(tile, with_metadata)
-            elif tile.coord is not None:
+            if tile.coord is not None and not self.is_cached(tile):
+                # missing or staled
                 uncached_tiles.append(tile)
         
         if uncached_tiles:
@@ -101,8 +132,7 @@ class TileManager(object):
     
     def remove_tile_coords(self, tile_coords):
         tiles = TileCollection(tile_coords)
-        for tile in tiles:
-            self.cache.remove(tile)
+        self.cache.remove_tiles(tiles)
     
     def creator(self):
         return TileCreator(self.cache, self.sources, self.grid, self.meta_grid, self)
@@ -118,10 +148,13 @@ class TileManager(object):
         """
         if isinstance(tile, tuple):
             tile = Tile(tile)
+        if tile.coord is None:
+            return True
         cached = self.cache.is_cached(tile)
         max_mtime = self.expire_timestamp(tile)
         if cached and max_mtime is not None:
-            stale = self.cache.timestamp_created(tile) < max_mtime
+            self.cache.load_tile_metadata(tile)
+            stale = tile.timestamp < max_mtime
             if stale:
                 cached = False
         return cached
@@ -223,9 +256,9 @@ class TileCreator(object):
                 source.as_buffer(self.tile_mgr.image_opts)
                 tile.source = source
                 tile = self.tile_mgr.apply_tile_filter(tile)
-                self.cache.store(tile)
+                self.cache.store_tile(tile)
             else:
-                self.cache.load(tile)
+                self.cache.load_tile(tile)
         return [tile]
     
     def _query_sources(self, query):
@@ -274,14 +307,12 @@ class TileCreator(object):
                 if not meta_tile_image: return []
                 splitted_tiles = split_meta_tiles(meta_tile_image, meta_tile.tile_patterns,
                                                   tile_size, self.tile_mgr.image_opts)
-                for splitted_tile in splitted_tiles:
-                    splitted_tile = self.tile_mgr.apply_tile_filter(splitted_tile)
-                    self.cache.store(splitted_tile)
+                splitted_tiles = map(self.tile_mgr.apply_tile_filter, splitted_tiles)
+                self.cache.store_tiles(splitted_tiles)
                 return splitted_tiles
         # else
         tiles = [Tile(coord) for coord in meta_tile.tiles]
-        for tile in tiles:
-            self.cache.load(tile)
+        self.cache.load_tiles(tiles)
         return tiles
         
 class Tile(object):
