@@ -28,69 +28,11 @@ from copy import deepcopy
 import logging
 log = logging.getLogger('mapproxy.config')
 
-from mapproxy.srs import SRS
-from mapproxy.util.ext.odict import odict
-from mapproxy.cache.file import FileCache, DummyCache
-from mapproxy.util.lock import SemLock
-from mapproxy.util.yaml import load_yaml_file, YAMLError
-from mapproxy.config.config import load_default_config
-from mapproxy.client.http import auth_data_from_url, HTTPClient
-from mapproxy.image.opts import ImageOptions, compatible_image_options, ImageFormat
-
-def loader(loaders, name):
-    """
-    Return named class/function from loaders map.
-    """
-    entry_point = loaders[name]
-    module_name, class_name = entry_point.split(':')
-    module = __import__(module_name, {}, {}, class_name)
-    return getattr(module, class_name)
-
-
-tile_filter_loaders = {
-    'watermark': 'mapproxy.tilefilter:WaterMarkTileFilter',
-    'pngquant': 'mapproxy.tilefilter:PNGQuantTileFilter',
-}
-
-def load_tile_filters():
-    filters = []
-    for key in tile_filter_loaders:
-        filters.append(loader(tile_filter_loaders, key))
-    filters.sort(key=lambda x: x.priority, reverse=True)
-    conf_keys = set()
-    for f in filters:
-        conf_keys.update(f.cache_conf_keys)
-    return filters, conf_keys
-
-tile_filters, tile_filter_conf_keys = load_tile_filters()
-del load_tile_filters
-
-
-import mapproxy.config
-from mapproxy.grid import tile_grid, resolution_range as _resolution_range
-from mapproxy.request.base import split_mime_type
-from mapproxy.request.wms import create_request
-from mapproxy.layer import (
-    CacheMapLayer, SRSConditional,
-    ResolutionConditional, map_extent_from_grid, merge_layer_extents
-)
-from mapproxy.client.tile import TileClient, TileURLTemplate
-from mapproxy.client.wms import WMSClient, WMSInfoClient, WMSLegendClient, WMSLegendURLClient
-from mapproxy.service.wms import WMSServer, WMSLayer, WMSGroupLayer
-from mapproxy.service.wmts import WMTSServer
-from mapproxy.service.tile import TileServer, TileLayer
-from mapproxy.service.kml import KMLServer
-from mapproxy.service.demo import DemoServer
-from mapproxy.service.ows import OWSServer
-from mapproxy.source import DebugSource, DummySource
-from mapproxy.source.wms import WMSSource, WMSInfoSource, WMSLegendSource
-from mapproxy.source.tile import TiledSource
-from mapproxy.cache.tile import TileManager
-from mapproxy.cache.legend import LegendCache
-from mapproxy.util import local_base_config, memoize
-from mapproxy.config.coverage import load_coverage
+from mapproxy.config import load_default_config, finish_base_config
 from mapproxy.config.spec import validate_mapproxy_conf
-from mapproxy.featureinfo import XSLTransformer, has_xslt_support
+from mapproxy.util import memoize
+from mapproxy.util.ext.odict import odict
+from mapproxy.util.yaml import load_yaml_file, YAMLError
 
 class ConfigurationError(Exception):
     pass
@@ -257,6 +199,7 @@ class ProxyConfiguration(object):
         self.services = ServiceConfiguration(self.configuration.get('services', {}), context=self)
     
     def configured_services(self):
+        from mapproxy.util import local_base_config
         with local_base_config(self.base_config):
             return self.services.services()
     
@@ -298,6 +241,8 @@ class ConfigurationBase(object):
 class GridConfiguration(ConfigurationBase):
     @memoize
     def tile_grid(self):
+        from mapproxy.grid import tile_grid
+        
         if 'base' in self.conf:
             base_grid_name = self.conf['base']
             conf = self.context.grids[base_grid_name].conf.copy()
@@ -349,7 +294,7 @@ class GlobalConfiguration(ConfigurationBase):
         self.base_config = load_default_config()
         self._copy_conf_values(self.conf, self.base_config)
         self.base_config.conf_base_dir = conf_base_dir
-        mapproxy.config.finish_base_config(self.base_config)
+        finish_base_config(self.base_config)
         
         self.image_options = ImageOptionsConfiguration(self.conf.get('image', {}), context)
     
@@ -424,6 +369,8 @@ class ImageOptionsConfiguration(ConfigurationBase):
             raise ConfigurationError('unknown encoding_options: %r' % options)
     
     def image_opts(self, image_conf, format):
+        from mapproxy.image.opts import ImageOptions
+        
         conf = {}
         if format in self.formats:
             conf = self.formats[format].copy()
@@ -493,21 +440,28 @@ class SourcesCollection(dict):
         if not layers:
             return source
 
-        if source.conf.get('type') not in ('wms', 'mapserver'):
-            raise ConfigurationError("found ':' in non-WMS source name: '%s'."
-                " tagged sources only supported for WMS" % key)
+        if source.conf.get('type') not in ('wms', 'mapserver', 'mapnik'):
+            raise ConfigurationError("found ':' in: '%s'."
+                " tagged sources only supported for WMS/Mapserver/Mapnik" % key)
 
+        uses_req = source.conf.get('type') != 'mapnik'
         source = deepcopy(source)
         
-        supported_layers = source.conf['req'].get('layers', [])
+        if uses_req:
+            supported_layers = source.conf['req'].get('layers', [])
+        else:
+            supported_layers = source.conf.get('layers', [])
         supported_layer_set = SourcesCollection.layer_set(supported_layers)
         layer_set = SourcesCollection.layer_set(layers)
 
         if supported_layer_set and not layer_set.issubset(supported_layer_set):
             raise ConfigurationError('layers (%s) not supported by source (%s)' % (
-                layers, supported_layers))
-
-        source.conf['req']['layers'] = layers
+                layers, ','.join(supported_layer_set)))
+        
+        if uses_req:
+            source.conf['req']['layers'] = layers
+        else:
+            source.conf['layers'] = layers
         return source
 
     def __contains__(self, key):
@@ -537,6 +491,7 @@ class SourceConfiguration(ConfigurationBase):
     @memoize
     def coverage(self):
         if not 'coverage' in self.conf: return None
+        from mapproxy.config.coverage import load_coverage
         return load_coverage(self.conf['coverage'])
     
     def image_opts(self, format=None):
@@ -545,6 +500,8 @@ class SourceConfiguration(ConfigurationBase):
         return self.context.globals.image_options.image_opts(self.conf.get('image', {}), format)
     
     def http_client(self, url):
+        from mapproxy.client.http import auth_data_from_url, HTTPClient
+        
         http_client = None
         url, (username, password) = auth_data_from_url(url)
         insecure = ssl_ca_certs = None
@@ -561,6 +518,7 @@ class SourceConfiguration(ConfigurationBase):
         return http_client, url
 
 def resolution_range(conf):
+    from mapproxy.grid import resolution_range as _resolution_range
     if 'min_res' in conf or 'max_res' in conf:
         return _resolution_range(min_res=conf.get('min_res'),
                                 max_res=conf.get('max_res'))
@@ -574,6 +532,10 @@ class WMSSourceConfiguration(SourceConfiguration):
     
     @staticmethod
     def static_legend_source(url, context):
+        from mapproxy.cache.legend import LegendCache
+        from mapproxy.client.wms import WMSLegendURLClient
+        from mapproxy.source.wms import WMSLegendSource
+        
         cache_dir = os.path.join(context.globals.get_path('cache.base_dir', {}),
                                  'legends')
         if url.startswith('file://') and not url.startswith('file:///'):
@@ -584,6 +546,7 @@ class WMSSourceConfiguration(SourceConfiguration):
         return WMSLegendSource([lg_client], legend_cache)
     
     def fi_xslt_transformer(self, conf, context):
+        from mapproxy.featureinfo import XSLTransformer, has_xslt_support
         fi_transformer = None
         fi_xslt = conf.get('featureinfo_xslt')
         if fi_xslt:
@@ -602,10 +565,16 @@ class WMSSourceConfiguration(SourceConfiguration):
         return SourceConfiguration.image_opts(self, format=format)
     
     def source(self, params=None):
+        from mapproxy.client.wms import WMSClient
+        from mapproxy.request.wms import create_request
+        from mapproxy.source.wms import WMSSource
+        from mapproxy.srs import SRS
+        
         if not self.conf.get('wms_opts', {}).get('map', True):
             return None
         
         if not self.context.seed and self.conf.get('seed_only'):
+            from mapproxy.source import DummySource
             return DummySource()
         
         if params is None: params = {}
@@ -624,6 +593,7 @@ class WMSSourceConfiguration(SourceConfiguration):
         concurrent_requests = self.context.globals.get_value('concurrent_requests', self.conf,
                                                         global_key='http.concurrent_requests')
         if concurrent_requests:
+            from mapproxy.util.lock import SemLock
             lock_dir = self.context.globals.get_path('cache.lock_dir', self.conf)
             url = urlparse.urlparse(self.conf['req']['url'])
             md5 = hashlib.md5(url.netloc)
@@ -653,6 +623,11 @@ class WMSSourceConfiguration(SourceConfiguration):
                          supported_formats=supported_formats or None)
     
     def fi_source(self, params=None):
+        from mapproxy.client.wms import WMSInfoClient
+        from mapproxy.request.wms import create_request
+        from mapproxy.source.wms import WMSInfoSource
+        from mapproxy.srs import SRS
+        
         if params is None: params = {}
         request_format = self.conf['req'].get('format')
         if request_format:
@@ -678,6 +653,11 @@ class WMSSourceConfiguration(SourceConfiguration):
         return fi_source
     
     def lg_source(self, params=None):
+        from mapproxy.cache.legend import LegendCache
+        from mapproxy.client.wms import WMSLegendClient
+        from mapproxy.request.wms import create_request
+        from mapproxy.source.wms import WMSLegendSource
+        
         if params is None: params = {}
         request_format = self.conf['req'].get('format')
         if request_format:
@@ -738,6 +718,7 @@ class MapnikSourceConfiguration(SourceConfiguration):
     
     def source(self, params=None):
         if not self.context.seed and self.conf.get('seed_only'):
+            from mapproxy.source import DummySource
             return DummySource()
         
         image_opts = self.image_opts()
@@ -746,6 +727,7 @@ class MapnikSourceConfiguration(SourceConfiguration):
         concurrent_requests = self.context.globals.get_value('concurrent_requests', self.conf,
                                                         global_key='http.concurrent_requests')
         if concurrent_requests:
+            from mapproxy.util.lock import SemLock
             lock_dir = self.context.globals.get_path('cache.lock_dir', self.conf)
             md5 = hashlib.md5(self.conf['mapfile'])
             lock_file = os.path.join(lock_dir, md5.hexdigest() + '.lck')
@@ -754,17 +736,25 @@ class MapnikSourceConfiguration(SourceConfiguration):
         coverage = self.coverage()
         res_range = resolution_range(self.conf)
         
+        layers = self.conf.get('layers', None)
+        if isinstance(layers, basestring):
+            layers = layers.split(',')
+        
         mapfile = self.context.globals.abspath(self.conf['mapfile'])
         from mapproxy.source.mapnik import MapnikSource
-        return MapnikSource(mapfile, image_opts=image_opts, coverage=coverage,
-                         res_range=res_range, lock=lock)
+        return MapnikSource(mapfile, layers=layers, image_opts=image_opts,
+            coverage=coverage, res_range=res_range, lock=lock)
 
 class TileSourceConfiguration(SourceConfiguration):
     source_type = ('tile',)
     defaults = {'origin': 'sw', 'grid': 'GLOBAL_MERCATOR'}
     
     def source(self, params=None):
+        from mapproxy.client.tile import TileClient, TileURLTemplate
+        from mapproxy.source.tile import TiledSource
+        
         if not self.context.seed and self.conf.get('seed_only'):
+            from mapproxy.source import DummySource
             return DummySource()
         
         if params is None: params = {}
@@ -789,6 +779,7 @@ class TileSourceConfiguration(SourceConfiguration):
 
 
 def file_ext(mimetype):
+    from mapproxy.request.base import split_mime_type
     _mime_class, format, _options = split_mime_type(mimetype)
     return format
 
@@ -797,6 +788,7 @@ class DebugSourceConfiguration(SourceConfiguration):
     required_keys = set('type'.split())
     
     def source(self, params=None):
+        from mapproxy.source import DebugSource
         return DebugSource()
 
 
@@ -821,11 +813,11 @@ class CacheConfiguration(ConfigurationBase):
             global_key='cache.base_dir')
         
     def _file_cache(self, grid_conf, file_ext):
-        if self.conf.get('disable_storage', False):
-            return DummyCache()
+        from mapproxy.cache.file import FileCache
         
         cache_dir = self.cache_dir()
         grid_conf.tile_grid() #create to resolve `base` in grid_conf.conf
+        directory_layout = self.conf.get('cache', {}).get('directory_layout', 'tc')
         suffix = grid_conf.conf['srs'].replace(':', '')
         cache_dir = os.path.join(cache_dir, self.conf['name'] + '_' + suffix)
         link_single_color_images = self.conf.get('link_single_color_images', False)
@@ -835,19 +827,43 @@ class CacheConfiguration(ConfigurationBase):
         
         lock_timeout = self.context.globals.get_value('http.client_timeout', {})
         
-        return FileCache(cache_dir, file_ext=file_ext,
+        return FileCache(cache_dir, file_ext=file_ext, directory_layout=directory_layout,
             lock_timeout=lock_timeout, link_single_color_images=link_single_color_images)
     
+    def _mbtiles_cache(self, grid_conf, file_ext):
+        from mapproxy.cache.mbtiles import MBTilesCache
+
+        filename = self.conf['cache'].get('filename')
+        if not filename:
+            filename = self.conf['name'] + '.mbtiles'
+        
+        if filename.startswith('.' + os.sep):
+            mbfile_path = self.context.globals.abspath(filename)
+        else:
+            mbfile_path = os.path.join(self.cache_dir(), filename)
+        return MBTilesCache(mbfile_path)
+
+    def _tile_cache(self, grid_conf, file_ext):
+        if self.conf.get('disable_storage', False):
+            from mapproxy.cache.dummy import DummyCache
+            return DummyCache()
+        
+        cache_type = self.conf.get('cache', {}).get('type', 'file')
+        return getattr(self, '_%s_cache' % cache_type)(grid_conf, file_ext)
+    
     def _tile_filter(self):
         filters = []
-        for tile_filter in tile_filters:
-            f = tile_filter().create_filter(self.conf, self.context)
-            if f is not None:
+        if 'watermark' in self.conf:
+            from mapproxy.tilefilter import create_watermark_filter
+            f = create_watermark_filter(self.conf, self.context)
+            if f:
                 filters.append(f)
         return filters
     
     @memoize
     def image_opts(self):
+        from mapproxy.image.opts import ImageFormat
+        
         format = None
         if 'format' not in self.conf.get('image', {}):
             format = self.conf.get('format') or self.conf.get('request_format')
@@ -861,6 +877,10 @@ class CacheConfiguration(ConfigurationBase):
         
     @memoize
     def caches(self):
+        from mapproxy.cache.tile import TileManager
+        from mapproxy.image.opts import compatible_image_options
+        from mapproxy.layer import map_extent_from_grid, merge_layer_extents
+        
         base_image_opts = self.image_opts()
         request_format = self.conf.get('request_format') or self.conf.get('format')
         caches = []
@@ -885,11 +905,14 @@ class CacheConfiguration(ConfigurationBase):
                 if source:
                     sources.append(source)
                     source_image_opts.append(source.image_opts)
-            assert sources, 'no sources configured for %s' % self.conf['name']
+            if not sources:
+                from mapproxy.source import DummySource
+                sources = [DummySource()]
+                source_image_opts.append(sources[0].image_opts)
             tile_grid = grid_conf.tile_grid()
             tile_filter = self._tile_filter()
             image_opts = compatible_image_options(source_image_opts, base_opts=base_image_opts)
-            cache = self._file_cache(grid_conf, image_opts.format.ext)
+            cache = self._tile_cache(grid_conf, image_opts.format.ext)
             mgr = TileManager(tile_grid, cache, sources, image_opts.format.ext,
                               image_opts=image_opts,
                               meta_size=meta_size, meta_buffer=meta_buffer,
@@ -904,6 +927,8 @@ class CacheConfiguration(ConfigurationBase):
     
     @memoize
     def map_layer(self):
+        from mapproxy.layer import CacheMapLayer, SRSConditional, ResolutionConditional
+        
         image_opts = self.image_opts()
         max_tile_limit = self.context.globals.get_value('max_tile_limit', self.conf,
             global_key='cache.max_tile_limit')
@@ -935,6 +960,8 @@ class CacheConfiguration(ConfigurationBase):
 class WMSLayerConfiguration(ConfigurationBase):
     @memoize
     def wms_layer(self):
+        from mapproxy.service.wms import WMSGroupLayer
+        
         layers = []
         this_layer = None
         
@@ -959,6 +986,8 @@ class WMSLayerConfiguration(ConfigurationBase):
 class LayerConfiguration(ConfigurationBase):
     @memoize
     def wms_layer(self):
+        from mapproxy.service.wms import WMSLayer
+        
         sources = []
         fi_sources = []
         lg_sources = []
@@ -1006,6 +1035,8 @@ class LayerConfiguration(ConfigurationBase):
     
     @memoize
     def tile_layers(self):
+        from mapproxy.service.tile import TileLayer
+        
         if len(self.conf.get('sources', [])) > 1: return [] #TODO
         
         tile_layers = []
@@ -1018,6 +1049,7 @@ class LayerConfiguration(ConfigurationBase):
                 md['name_path'] = (self.conf['name'], grid.srs.srs_code.replace(':', '').upper())
                 md['name_internal'] = md['name_path'][0] + '_' + md['name_path'][1]
                 md['format'] = self.context.caches[cache_name].image_opts().format
+                md['extent'] = extent
             
                 tile_layers.append(TileLayer(self.conf['name'], self.conf['title'],
                                              md, cache_source))
@@ -1026,6 +1058,7 @@ class LayerConfiguration(ConfigurationBase):
         
 
 def fi_xslt_transformers(conf, context):
+    from mapproxy.featureinfo import XSLTransformer, has_xslt_support
     fi_transformers = {}
     fi_xslt = conf.get('featureinfo_xslt')
     if fi_xslt:
@@ -1050,6 +1083,7 @@ class ServiceConfiguration(ConfigurationBase):
                 services.append(creator(service_conf or {}))
         
         if ows_services:
+            from mapproxy.service.ows import OWSServer
             services.append(OWSServer(ows_services))
         return services
     
@@ -1062,6 +1096,8 @@ class ServiceConfiguration(ConfigurationBase):
         return layers
     
     def kml_service(self, conf):
+        from mapproxy.service.kml import KMLServer
+        
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
         max_tile_age = self.context.globals.get_value('tiles.expires_hours')
@@ -1070,6 +1106,8 @@ class ServiceConfiguration(ConfigurationBase):
         return KMLServer(layers, md, max_tile_age=max_tile_age)
     
     def tms_service(self, conf):
+        from mapproxy.service.tile import TileServer
+        
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
         max_tile_age = self.context.globals.get_value('tiles.expires_hours')
@@ -1078,12 +1116,16 @@ class ServiceConfiguration(ConfigurationBase):
         return TileServer(layers, md, max_tile_age=max_tile_age)
     
     def wmts_service(self, conf):
+        from mapproxy.service.wmts import WMTSServer
+        
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
         layers = self.tile_layers(conf)
         return WMTSServer(layers, md)
     
     def wms_service(self, conf):
+        from mapproxy.service.wms import WMSServer
+        
         md = conf.get('md', {})
         tile_layers = self.tile_layers(conf)
         attribution = conf.get('attribution')
@@ -1102,6 +1144,7 @@ class ServiceConfiguration(ConfigurationBase):
         image_formats = {}
         for format in image_formats_names:
             if format.startswith('image/'):
+                from mapproxy.image.opts import ImageOptions
                 opts = ImageOptions(format=format)
             else:
                 # lookup custom format
@@ -1123,6 +1166,8 @@ class ServiceConfiguration(ConfigurationBase):
         return server
 
     def demo_service(self, conf):
+        from mapproxy.service.demo import DemoServer
+        
         md = self.context.services.conf.get('wms', {}).get('md', {}).copy()
         md.update(conf.get('md', {}))
         layers = odict()
