@@ -1,36 +1,36 @@
 # This file is part of the MapProxy project.
 # Copyright (C) 2011 Omniscale <http://omniscale.de>
 # 
-# This program is free software: you can redistribute it and/or modify
-# it under the terms of the GNU Affero General Public License as published by
-# the Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
+# Licensed under the Apache License, Version 2.0 (the "License");
+# you may not use this file except in compliance with the License.
+# You may obtain a copy of the License at
 # 
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Affero General Public License for more details.
+#    http://www.apache.org/licenses/LICENSE-2.0
 # 
-# You should have received a copy of the GNU Affero General Public License
-# along with this program.  If not, see <http://www.gnu.org/licenses/>.
+# Unless required by applicable law or agreed to in writing, software
+# distributed under the License is distributed on an "AS IS" BASIS,
+# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+# See the License for the specific language governing permissions and
+# limitations under the License.
 
 from __future__ import with_statement
 
-import os
 import sys
 import time
 import operator
 
-import yaml
+from mapproxy.config.loader import ConfigurationError
 from mapproxy.config.coverage import load_coverage
 from mapproxy.srs import SRS
-from mapproxy.util import memoize, timestamp_from_isodate, timestamp_before, local_base_config
+from mapproxy.util import memoize, timestamp_from_isodate, timestamp_before
 from mapproxy.util.geom import MultiCoverage, BBOXCoverage
+from mapproxy.util.yaml import load_yaml_file, YAMLError
 from mapproxy.seed.util import bidict
 from mapproxy.seed.seeder import SeedTask, CleanupTask
+from mapproxy.seed.spec import validate_seed_conf
 
 
-class SeedConfigurationError(Exception):
+class SeedConfigurationError(ConfigurationError):
     pass
 
 
@@ -41,14 +41,24 @@ class SeedConfigurationError(Exception):
 #             ' required for polygon/ogr seed areas')
 
 
+import logging
+log = logging.getLogger('mapproxy.seed.config')
+
 def load_seed_tasks_conf(seed_conf_filename, mapproxy_conf):
-    with open(seed_conf_filename) as f:
-        conf = yaml.load(f)
+    try:
+        conf = load_yaml_file(seed_conf_filename)
+    except YAMLError, ex:
+        raise SeedConfigurationError(ex)
     
     if 'views' in conf:
         # TODO: deprecate old config
         seed_conf = LegacySeedingConfiguration(conf, mapproxy_conf=mapproxy_conf)
     else:
+        errors, informal_only = validate_seed_conf(conf)
+        for error in errors:
+            log.warn(error)
+        if not informal_only:
+            raise SeedConfigurationError('invalid configuration')
         seed_conf = SeedingConfiguration(conf, mapproxy_conf=mapproxy_conf)
     return seed_conf
 
@@ -134,6 +144,9 @@ class SeedingConfiguration(object):
     
     def cache(self, cache_name):
         cache = {}
+        if cache_name not in self.mapproxy_conf.caches:
+            raise SeedConfigurationError('cache %s not found. available caches: %s' % (
+                cache_name, ','.join(self.mapproxy_conf.caches.keys())))
         for tile_grid, extent, tile_mgr in self.mapproxy_conf.caches[cache_name].caches():
             grid_name = self.grids[tile_grid]
             cache[grid_name] = tile_mgr
