@@ -14,7 +14,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from __future__ import division
+from __future__ import division, with_statement
 
 import math
 import time
@@ -183,8 +183,11 @@ class TileLayer(object):
             raise RequestError('The requested tile is outside the bounding box'
                                ' of the tile map.', request=tile_request,
                                code='TileOutOfRange')
-        if tile_request.origin == 'nw':
+        if tile_request.origin == 'nw' and self.grid.origin not in ('ul', 'nw'):
             tile_coord = self.grid.flip_tile_coord(tile_coord)
+        elif tile_request.origin == 'sw' and self.grid.origin not in ('ll', 'sw', None):
+            tile_coord = self.grid.flip_tile_coord(tile_coord)
+
         return tile_coord
     
     def empty_response(self):
@@ -201,7 +204,8 @@ class TileLayer(object):
                                code='InvalidParameterValue')
         tile_coord = self._internal_tile_coord(tile_request, use_profiles=use_profiles)
         try:
-            tile = self.tile_manager.load_tile_coord(tile_coord, with_metadata=True)
+            with self.tile_manager.session():
+                tile = self.tile_manager.load_tile_coord(tile_coord, with_metadata=True)
             if tile.source is None: return self.empty_response()
             return TileResponse(tile)
         except SourceError, e:
